package data.hullmods;

import com.fs.starfarer.api.combat.BaseHullMod;
import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipAPI.HullSize;
import com.fs.starfarer.api.combat.FluxTrackerAPI;

public class GDLM_BufferFlurry extends BaseHullMod {
	private static final float AMMO_SPREAD_DECREASE = 15f;
	private static final float ARMOR_DAMAGE_DECREASE = 10f;
	private static final String ID = "buffer_flurry";

	@Override
	public void applyEffectsAfterShipCreation(ShipAPI ship, java.lang.String id) {
		MutableShipStatsAPI stats = ship.getMutableStats();

		stats.getRecoilPerShotMult().modifyPercent(id, -AMMO_SPREAD_DECREASE);
	}

	@Override
	public void advanceInCombat(ShipAPI ship, float amount) {
		MutableShipStatsAPI stats = ship.getMutableStats();
		FluxTrackerAPI flux = ship.getFluxTracker();

		if (!flux.isOverloaded() && !flux.isVenting()) {
			stats.getArmorDamageTakenMult().modifyPercent(ID, ARMOR_DAMAGE_DECREASE);
		} else {
			stats.getArmorDamageTakenMult().unmodify(ID);
		}
	}

	@Override
	public String getDescriptionParam(int index, HullSize hullSize) {
		if (index == 0) return (int) ARMOR_DAMAGE_DECREASE + "%";
		if (index == 1) return (int) AMMO_SPREAD_DECREASE + "%";
		return null;
	}

	@Override
	public boolean isApplicableToShip(ShipAPI ship) {
		return ship != null && ship.getHullSpec().getHullId().startsWith("GL_") && ship.getVariant().getHullMods().contains("GDLM_ComplexFlurry") && !ship.getVariant().getHullMods().contains("GDLM_EnergyFlurry");
	}

	@Override
	public String getUnapplicableReason(ShipAPI ship) {
		if (!ship.getHullSpec().getHullId().startsWith("GL_")) return "只能安装在古达兰慕的舰船上";
		if (!ship.getVariant().getHullMods().contains("GDLM_ComplexFlurry")) return "未安装复合浆体夹层";
		if (ship.getVariant().getHullMods().contains("GDLM_EnergyFlurry")) return "与能量导流浆体冲突";
		return null;
	}

}
