package data.hullmods;

import com.fs.starfarer.api.combat.BaseHullMod;
import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI.HullSize;
import com.fs.starfarer.api.combat.ShipAPI;

public class GDLM_EnergyFlurry extends BaseHullMod {
	private static final float VELOCITY_DECREASE = 10f;
	private static final float WEAPON_FLUX_DECREASE = 15f;
	private static final float VENT_BONUS = 20f;

	@Override
	public void applyEffectsBeforeShipCreation(HullSize hullSize, MutableShipStatsAPI stats, java.lang.String id) {
		stats.getMaxSpeed().modifyMult(id, 1f - VELOCITY_DECREASE * 0.01f);
		stats.getAcceleration().modifyMult(id, 1f - VELOCITY_DECREASE * 0.01f);
		stats.getDeceleration().modifyMult(id, 1f - VELOCITY_DECREASE * 0.01f);
		stats.getMaxTurnRate().modifyMult(id, 1f - VELOCITY_DECREASE * 0.01f);
		stats.getTurnAcceleration().modifyMult(id, 1f - VELOCITY_DECREASE * 0.01f);

		stats.getEnergyWeaponFluxCostMod().modifyMult(id, 1f - WEAPON_FLUX_DECREASE * 0.01f);
		stats.getBallisticWeaponFluxCostMod().modifyMult(id, 1f - WEAPON_FLUX_DECREASE * 0.01f);
		stats.getMissileWeaponFluxCostMod().modifyMult(id, 1f - WEAPON_FLUX_DECREASE * 0.01f);

		stats.getVentRateMult().modifyPercent(id, VENT_BONUS);
	}

	@Override
	public String getDescriptionParam(int index, HullSize hullSize) {
		if (index == 0) return (int) VELOCITY_DECREASE + "%";
		if (index == 1) return (int) WEAPON_FLUX_DECREASE + "%";
		if (index == 2) return (int) VENT_BONUS + "%";
		return null;
	}

	@Override
	public boolean isApplicableToShip(ShipAPI ship) {
		return ship != null && ship.getHullSpec().getHullId().startsWith("GL_") && ship.getVariant().getHullMods().contains("GDLM_ComplexFlurry") && !ship.getVariant().getHullMods().contains("GDLM_BufferFlurry");
	}

	@Override
	public String getUnapplicableReason(ShipAPI ship) {
		if (!ship.getHullSpec().getHullId().startsWith("GL_")) return "只能安装在古达兰慕的舰船上";
		if (!ship.getVariant().getHullMods().contains("GDLM_ComplexFlurry")) return "未安装复合浆体夹层";
		if (ship.getVariant().getHullMods().contains("GDLM_BufferFlurry")) return "与缓冲剪切浆体冲突";
		return null;
	}
}
