package data.hullmods;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.combat.BaseHullMod;
import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipAPI.HullSize;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.impl.campaign.DModManager;

import java.util.*;

public class GDLM_GenerateFuel extends BaseHullMod {

	float dayPassed = 0f;

	static final int GENERATE_INTERVAL = 1;//by day
	static final float CR_THRESHOLD = 0.65f;//cr 界限，低于它停止产出
	static final int D_MOD_THRESHOLD = 2;//dMod 界限， 多于它停止产出
	static final float PUNISH_RATE = 0.6f;//相加惩罚 1ship = 1, 2ships = 1 + 1*0.8, 3ships = 1 + 1*0.8 + 1*0.8*0.8
	static final Map GENERATE_SPEED = new HashMap();//不同尺寸舰体每天生产的量
	static {
		GENERATE_SPEED.put(HullSize.FIGHTER, 0f);
		GENERATE_SPEED.put(HullSize.FRIGATE, 3f);
		GENERATE_SPEED.put(HullSize.DESTROYER, 6f);
		GENERATE_SPEED.put(HullSize.CRUISER, 12f);
		GENERATE_SPEED.put(HullSize.CAPITAL_SHIP, 28f);
	}

	@Override
	public void applyEffectsBeforeShipCreation(ShipAPI.HullSize hullSize, MutableShipStatsAPI stats, String id) {

	}

	@Override
	public void advanceInCampaign(FleetMemberAPI member, float amount) {
		//time check
		if (Global.getSector().isPaused()) return;

		CampaignFleetAPI fleet = member.getFleetData().getFleet();
		if (fleet == null || fleet != Global.getSector().getPlayerFleet()) return;

		dayPassed += Global.getSector().getClock().convertToDays(amount);
		if (dayPassed < GENERATE_INTERVAL) return;
		dayPassed -= GENERATE_INTERVAL;

		//hyperspace check
		if (fleet.isInHyperspace()) return;
		if (!fleet.isInCurrentLocation()) return;

		float[] data = getFuelToGenerate(fleet);
		float capacity = Math.max(fleet.getCargo().getMaxFuel() - fleet.getCargo().getFuel(), 0f);
		float fuelToAdd = data[0] / data[1];
		fleet.getCargo().addFuel(Math.min(fuelToAdd, capacity));
	}

	float[] getFuelToGenerate(CampaignFleetAPI fleet) {
		if (fleet == null) return new float[]{0, 1};
		List<FleetMemberAPI> frigate = new ArrayList<>();
		List<FleetMemberAPI> destroyer = new ArrayList<>();
		List<FleetMemberAPI> cruiser = new ArrayList<>();
		List<FleetMemberAPI> capital = new ArrayList<>();
		for (FleetMemberAPI member : fleet.getFleetData().getMembersListCopy()) {
			if (!member.isMothballed() && member.getRepairTracker().getCR() > CR_THRESHOLD) {
				if (DModManager.getNumDMods(member.getVariant()) <= D_MOD_THRESHOLD) {
					if (member.getVariant().hasHullMod("GDLM_GenerateFuel"))
					{
						if(member.getHullSpec().getHullSize()==HullSize.CAPITAL_SHIP)capital.add(member);
						if(member.getHullSpec().getHullSize()==HullSize.CRUISER)cruiser.add(member);
						if(member.getHullSpec().getHullSize()==HullSize.DESTROYER)destroyer.add(member);
						if(member.getHullSpec().getHullSize()==HullSize.FRIGATE)frigate.add(member);
					}
				}
			}
		}

		int numOfShip = 0;
		float fuelToGenerate = 0f;
		for (FleetMemberAPI s : capital) {
			float mult = (float) Math.pow(PUNISH_RATE, (double) numOfShip);
			fuelToGenerate += (Float) GENERATE_SPEED.get(s.getHullSpec().getHullSize()) * mult;
			numOfShip ++;
		}
		for (FleetMemberAPI s : cruiser) {
			float mult = (float) Math.pow(PUNISH_RATE, (double) numOfShip);
			fuelToGenerate += (Float) GENERATE_SPEED.get(s.getHullSpec().getHullSize()) * mult;
			numOfShip ++;
		}
		for (FleetMemberAPI s : destroyer) {
			float mult = (float) Math.pow(PUNISH_RATE, (double) numOfShip);
			fuelToGenerate += (Float) GENERATE_SPEED.get(s.getHullSpec().getHullSize()) * mult;
			numOfShip ++;
		}
		for (FleetMemberAPI s : frigate) {
			float mult = (float) Math.pow(PUNISH_RATE, (double) numOfShip);
			fuelToGenerate += (Float) GENERATE_SPEED.get(s.getHullSpec().getHullSize()) * mult;
			numOfShip ++;
		}
		numOfShip = Math.max(1,numOfShip);
		return new float[]{fuelToGenerate, numOfShip};
	}

	@Override
	public String getDescriptionParam(int index, ShipAPI.HullSize hullSize) {
		float[] data = getFuelToGenerate(Global.getSector().getPlayerFleet());
		if (index == 0) {
			return ((Float) GENERATE_SPEED.get(HullSize.FRIGATE)).intValue() + "";
		}
		if (index == 1) {
			return ((Float) GENERATE_SPEED.get(HullSize.DESTROYER)).intValue() + "";
		}
		if (index == 2) {
			return ((Float) GENERATE_SPEED.get(HullSize.CRUISER)).intValue() + "";
		}
		if (index == 3) {
			return ((Float) GENERATE_SPEED.get(HullSize.CAPITAL_SHIP)).intValue() + "";
		}
		if (index == 4) {
			return (int) (100 - PUNISH_RATE * 100) + "%";
		}
		if (index == 5) {
			return (int) data[0] + "";
		}
		if (index == 6) {
			return (int) (CR_THRESHOLD * 100) + "%";
		}
		if (index == 7) {
			return D_MOD_THRESHOLD + "";
		}

		return null;
	}

}

	 
   

       


