package data.missions.Be;

import com.fs.starfarer.api.combat.BaseEveryFrameCombatPlugin;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.fleet.FleetGoal;
import com.fs.starfarer.api.fleet.FleetMemberType;
import com.fs.starfarer.api.mission.FleetSide;
import com.fs.starfarer.api.mission.MissionDefinitionAPI;
import com.fs.starfarer.api.mission.MissionDefinitionPlugin;

import java.util.List;

public class MissionDefinition implements MissionDefinitionPlugin {

	@Override
	public void defineMission(MissionDefinitionAPI api) {

		// Set up the fleets
		api.initFleet(FleetSide.PLAYER, "GLS", FleetGoal.ATTACK, false, 10);
		api.initFleet(FleetSide.ENEMY, "虫潮", FleetGoal.ATTACK, true, 10);

		// Set a blurb for each fleet
		api.setFleetTagline(FleetSide.PLAYER, "途径3-7聚落的征召舰队");
		api.setFleetTagline(FleetSide.ENEMY, "死亡集合的游弋舰队");

		// These show up as items in the bulleted list under 
		// "Tactical Objectives" on the mission detail screen
		api.addBriefingItem("彻底击溃敌人");
		api.addBriefingItem("有效利用你的重型巡洋舰保护坚壳级");
		api.addBriefingItem("请配合GLPlus进行本战役");

		// Set up the player's fleet
		api.addToFleet(FleetSide.PLAYER, "GL_Shell_Elite", FleetMemberType.SHIP, "GLS Light star", true);

		api.addToFleet(FleetSide.PLAYER, "GL_Annihilation_Assault", FleetMemberType.SHIP, "GLS Rays", false);
		api.addToFleet(FleetSide.PLAYER, "GL_Annihilation_Assault", FleetMemberType.SHIP, "GLS space", false);

		api.addToFleet(FleetSide.PLAYER, "GL_Shoulderpad_Assault", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "GL_Shoulderpad_Assault", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "GL_Chelated_Eilte", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "GL_Coral_Assault", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "GL_Position_Attack", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "GL_Position_Attack", FleetMemberType.SHIP, false);

		api.addToFleet(FleetSide.PLAYER, "GL_Telescope_Support", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "GL_Telescope_Support", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "GL_Telescope_Support", FleetMemberType.SHIP, false);

		//api.addToFleet(FleetSide.PLAYER, "onslaught_Standard", FleetMemberType.SHIP, "TTS Invincible", true, CrewXPLevel.ELITE);

		// Mark player flagship as essential

		// Set up the enemy fleet
		api.addToFleet(FleetSide.ENEMY, "Queen_variant", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "Queen_variant", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "Tianniu_variant", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "Tianniu_variant", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "Tianniu_variant", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "Sheath_variant", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "Sheath_variant", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "Sheath_variant", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "Sheath_variant", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "Sheath_variant", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "Sheath_variant", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "Sheath_variant", FleetMemberType.SHIP, false);

		// Set up the map.
		float width = 20000f;
		float height = 18000f;
		api.initMap((float) -width / 2f, (float) width / 2f, (float) -height / 2f, (float) height / 2f);

		float minX = -width / 2;
		float minY = -height / 2;

		for (int i = 0; i < 15; i++) {
			float x = (float) Math.random() * width - width / 2;
			float y = (float) Math.random() * height - height / 2;
			float radius = 100f + (float) Math.random() * 900f;
			api.addNebula(x, y, radius);
		}
		api.setBackgroundSpriteName("graphics/backgrounds/hyperspace1.jpg");
		//api.setBackgroundSpriteName("graphics/backgrounds/background6.jpg");

		//system.setBackgroundTextureFilename("graphics/backgrounds/background6.jpg");
		//api.setBackgroundSpriteName();

		// Add an asteroid field going diagonally across the
		// battlefield, 2000 pixels wide, with a maximum of 
		// 100 asteroids in it.
		// 20-70 is the range of asteroid speeds.
		api.addAsteroidField(0f, 0f, (float) Math.random() * 360f, width, 20f, 70f, 100);

		api.addPlugin(new BaseEveryFrameCombatPlugin() {
			@Override
			public void advance(float amount, List events) {
			}

			@Override
			public void init(CombatEngineAPI engine) {
				engine.getContext().setStandoffRange(10000f);
			}
		});
	}

}






