package data.missions.Exotic;

import java.util.List;

import com.fs.starfarer.api.combat.BaseEveryFrameCombatPlugin;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.fleet.FleetGoal;
import com.fs.starfarer.api.fleet.FleetMemberType;
import com.fs.starfarer.api.mission.FleetSide;
import com.fs.starfarer.api.mission.MissionDefinitionAPI;
import com.fs.starfarer.api.mission.MissionDefinitionPlugin;

public class MissionDefinition implements MissionDefinitionPlugin {

	@Override
	public void defineMission(MissionDefinitionAPI api) {

		// Set up the fleets
		api.initFleet(FleetSide.PLAYER, "ISS", FleetGoal.ATTACK, false, 10);
		api.initFleet(FleetSide.ENEMY, "HSS", FleetGoal.ATTACK, true, 10);

		// Set a blurb for each fleet
		api.setFleetTagline(FleetSide.PLAYER, "我最宝贵的贸易商队");
		api.setFleetTagline(FleetSide.ENEMY, "可恶的海盗舰队");

		// These show up as items in the bulleted list under 
		// "Tactical Objectives" on the mission detail screen
		api.addBriefingItem("消灭所有海盗");
		api.addBriefingItem("利用好刚刚到手的外星人护卫舰");

		// Set up the player's fleet
		api.addToFleet(FleetSide.PLAYER, "GL_Beetle_Support", FleetMemberType.SHIP, true);
		api.addToFleet(FleetSide.PLAYER, "GL_Bullet_Assault", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "vigilance_Strike", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "kite_Standard", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "colossus_Standard", FleetMemberType.SHIP, false);

		//api.addToFleet(FleetSide.PLAYER, "onslaught_Standard", FleetMemberType.SHIP, "TTS Invincible", true, CrewXPLevel.ELITE);

		// Mark player flagship as essential

		// Set up the enemy fleet

		api.addToFleet(FleetSide.ENEMY, "enforcer_Assault", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "cerberus_d_pirates_Shielded", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "cerberus_d_pirates_Shielded", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "cerberus_Overdriven", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "kite_pirates_Raider", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "kite_pirates_Raider", FleetMemberType.SHIP, false);

		// Set up the map.
		float width = 24000f;
		float height = 18000f;
		api.initMap((float) -width / 2f, (float) width / 2f, (float) -height / 2f, (float) height / 2f);

		float minX = -width / 2;
		float minY = -height / 2;

		for (int i = 0; i < 15; i++) {
			float x = (float) Math.random() * width - width / 2;
			float y = (float) Math.random() * height - height / 2;
			float radius = 100f + (float) Math.random() * 900f;
			api.addNebula(x, y, radius);
		}
		api.setBackgroundSpriteName("graphics/backgrounds/hyperspace1.jpg");
		//api.setBackgroundSpriteName("graphics/backgrounds/background6.jpg");

		//system.setBackgroundTextureFilename("graphics/backgrounds/background6.jpg");
		//api.setBackgroundSpriteName();

		// Add an asteroid field going diagonally across the
		// battlefield, 2000 pixels wide, with a maximum of 
		// 100 asteroids in it.
		// 20-70 is the range of asteroid speeds.
		api.addAsteroidField(0f, 0f, (float) Math.random() * 360f, width, 20f, 70f, 100);

		api.addPlugin(new BaseEveryFrameCombatPlugin() {
			@Override
			public void advance(float amount, List events) {
			}

			@Override
			public void init(CombatEngineAPI engine) {
				engine.getContext().setStandoffRange(10000f);
			}
		});
	}

}






