package data.missions.SecretTreasure;

import java.util.List;

import com.fs.starfarer.api.combat.BaseEveryFrameCombatPlugin;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.fleet.FleetGoal;
import com.fs.starfarer.api.fleet.FleetMemberType;
import com.fs.starfarer.api.mission.FleetSide;
import com.fs.starfarer.api.mission.MissionDefinitionAPI;
import com.fs.starfarer.api.mission.MissionDefinitionPlugin;

public class MissionDefinition implements MissionDefinitionPlugin {

	@Override
	public void defineMission(MissionDefinitionAPI api) {

		// Set up the fleets
		api.initFleet(FleetSide.PLAYER, "GLS", FleetGoal.ATTACK, false, 10);
		api.initFleet(FleetSide.ENEMY, "TTS", FleetGoal.ATTACK, true, 10);

		// Set a blurb for each fleet
		api.setFleetTagline(FleetSide.PLAYER, "编制不全的第六十二前线防御舰队");
		api.setFleetTagline(FleetSide.ENEMY, "速子科技封锁舰队");

		// These show up as items in the bulleted list under 
		// "Tactical Objectives" on the mission detail screen
		api.addBriefingItem("歼灭所有人类舰船");
		api.addBriefingItem("注意对方行动诡异的相位舰船");

		// Set up the player's fleet
		api.addToFleet(FleetSide.PLAYER, "GL_Revival_Elite", FleetMemberType.SHIP, "GLS Light star", true);
		api.addToFleet(FleetSide.PLAYER, "GL_Spine_Support", FleetMemberType.SHIP, "GLS Rays", false);
		api.addToFleet(FleetSide.PLAYER, "GL_Haven_Support", FleetMemberType.SHIP, "GLS space", false);
		api.addToFleet(FleetSide.PLAYER, "GL_Shoulderpad_Assault", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "GL_Shoulderpad_Assault", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "GL_Beetle_Support", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "GL_Beetle_Support", FleetMemberType.SHIP, false);

		//api.addToFleet(FleetSide.PLAYER, "onslaught_Standard", FleetMemberType.SHIP, "TTS Invincible", true, CrewXPLevel.ELITE);

		// Mark player flagship as essential

		// Set up the enemy fleet   aurora_Strike  brawler_tritachyon_Standard  omen_PD

		//api.addToFleet(FleetSide.ENEMY, "doom_Strike", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "aurora_Strike", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "medusa_CS", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "medusa_Attack", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "tempest_Attack", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "tempest_Attack", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "afflictor_Strike", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "afflictor_Strike", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "omen_PD", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "brawler_tritachyon_Standard", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "brawler_tritachyon_Standard", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "wolf_Assault", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "wolf_Assault", FleetMemberType.SHIP, false);

		// Set up the map.
		float width = 24000f;
		float height = 18000f;
		api.initMap((float) -width / 2f, (float) width / 2f, (float) -height / 2f, (float) height / 2f);

		float minX = -width / 2;
		float minY = -height / 2;

		for (int i = 0; i < 15; i++) {
			float x = (float) Math.random() * width - width / 2;
			float y = (float) Math.random() * height - height / 2;
			float radius = 100f + (float) Math.random() * 900f;
			api.addNebula(x, y, radius);
		}
		api.setBackgroundSpriteName("graphics/backgrounds/hyperspace1.jpg");
		//api.setBackgroundSpriteName("graphics/backgrounds/background6.jpg");

		//system.setBackgroundTextureFilename("graphics/backgrounds/background6.jpg");
		//api.setBackgroundSpriteName();

		// Add an asteroid field going diagonally across the
		// battlefield, 2000 pixels wide, with a maximum of 
		// 100 asteroids in it.
		// 20-70 is the range of asteroid speeds.
		api.addAsteroidField(0f, 0f, (float) Math.random() * 360f, width, 20f, 70f, 100);

		api.addPlugin(new BaseEveryFrameCombatPlugin() {
			@Override
			public void advance(float amount, List events) {
			}

			@Override
			public void init(CombatEngineAPI engine) {
				engine.getContext().setStandoffRange(10000f);
			}
		});
	}

}






