package data.scripts;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.*;
import com.fs.starfarer.api.impl.combat.BaseShipSystemScript;
import org.lazywizard.lazylib.combat.AIUtils;

import java.awt.*;
import java.util.HashMap;
import java.util.Map;

public class GDLMBurstEMPStats extends BaseShipSystemScript {

	public static final float EMP_DELAY = 0.55f;
	public static final float RANGE_BOUNUS = 200f;
	public static final float FLUX_LEVEL = 0.5f;
	public static final String ID = "GDLM_burst_emp";

	@Override
	public void apply(MutableShipStatsAPI stats, String id, State state, float effectLevel) {
		ShipAPI ship = (ShipAPI) stats.getEntity();
		CombatEngineAPI engine = Global.getCombatEngine();
		float amount = engine.getElapsedInLastFrame();

		if (!engine.getCustomData().containsKey(ID)) {
			engine.getCustomData().put(ID, new HashMap());
		}
		Map<ShipAPI, theState> shipsMap = (Map) engine.getCustomData().get(ID);
		if (engine.isPaused() || !engine.isEntityInPlay(ship) || !ship.isAlive()) {
			if (!ship.isAlive() && shipsMap.containsKey(ship)) {
				shipsMap.remove(ship);
			}
			return;
		}
		if (!shipsMap.containsKey(ship)) {
			shipsMap.put(ship, new theState());
		} else {
			theState data = (theState) shipsMap.get(ship);
			if (effectLevel >= 1f) {
				if (data.clock < 0.2f) {
					data.clock += amount;
				} else {
					generateArc(ship);
					data.clock = 0f;
				}
			}
		}
	}

	@Override
	public void unapply(MutableShipStatsAPI stats, String id) {
	}

	@Override
	public StatusData getStatusData(int index, State state, float effectLevel) {
		String statusData = "";
		if (effectLevel < 1f && state != State.OUT) {
			statusData = "Charging " + (int) (effectLevel * 100) + "%";
		} else {
			statusData = "Active";
		}

		if (index == 0) {
			return new StatusData(statusData, false);
		}
		return null;
	}

	protected final static class theState {
		float clock;

		protected theState() {
			clock = 0f;
		}
	}

	public void generateArc(ShipAPI ship) {
		CombatEngineAPI engine = Global.getCombatEngine();
		CombatEntityAPI target = getNearestEnemyEntity((CombatEntityAPI) ship);
		if (target == null) return;
		engine.spawnEmpArc(ship, ship.getLocation(), ship, target, DamageType.ENERGY, 100, 500, 500, "system_emp_emitter_impact", 20f, new Color(255, 150, 255, 200), new Color(255, 175, 255, 255));
	}

	public CombatEntityAPI getNearestEnemyEntity(CombatEntityAPI entity) {
		ShipAPI enemyShip = AIUtils.getNearestEnemy(entity);
		MissileAPI enemyMissile = AIUtils.getNearestEnemyMissile(entity);
		if (enemyShip == null || enemyMissile == null) {
			if (enemyShip == null && enemyMissile == null) return null;
			if (enemyShip == null) return (CombatEntityAPI) enemyMissile;
			return (CombatEntityAPI) enemyShip;
		}
		float shipRange = getRangeOfTwoEntities(entity, enemyShip);
		float missileRange = getRangeOfTwoEntities(entity, enemyMissile);
		if (shipRange <= missileRange) {
			return (CombatEntityAPI) enemyShip;
		} else {
			return (CombatEntityAPI) enemyMissile;
		}
	}

	public float getRangeOfTwoEntities(CombatEntityAPI entity1, CombatEntityAPI entity2) {
		float dX = (entity1.getLocation().getX() - entity2.getLocation().getX());
		float dY = (entity1.getLocation().getY() - entity2.getLocation().getY());
		return (float) Math.sqrt((double) (dX * dX + dY * dY));
	}

}
