package data.scripts;

import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.impl.combat.BaseShipSystemScript;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.FluxTrackerAPI;
import com.fs.starfarer.api.plugins.ShipSystemStatsScript;

public class GDLMFluxChargerStats extends BaseShipSystemScript {

	public static final float MAX_SPEED = 7.6f;
	public static final float ACC = 10f;

	@Override
	public void apply(MutableShipStatsAPI stats, String id, State state, float effectLevel) {
		ShipAPI ship = (ShipAPI) stats.getEntity();
		if (ship == null) {
			return;
		}
		if (state == ShipSystemStatsScript.State.OUT) {
			stats.getMaxSpeed().unmodify(id);
			stats.getMaxTurnRate().unmodify(id);
		} else {
			FluxTrackerAPI flux = ship.getFluxTracker();
			float fluxPercent = flux.getCurrFlux() / flux.getMaxFlux() * 100f;
			stats.getMaxSpeed().modifyPercent(id, fluxPercent * MAX_SPEED / 5f * effectLevel);
			stats.getAcceleration().modifyPercent(id, fluxPercent * ACC / 5f * effectLevel);
			stats.getDeceleration().modifyPercent(id, fluxPercent * ACC / 5f * effectLevel);
			stats.getMaxTurnRate().modifyPercent(id, fluxPercent * ACC / 5f * effectLevel);
			stats.getTurnAcceleration().modifyPercent(id, fluxPercent * ACC / 5f * effectLevel);
		}
	}

	@Override
	public void unapply(MutableShipStatsAPI stats, String id) {
		ShipAPI ship = (ShipAPI) stats.getEntity();
		if (ship == null) {
			return;
		}
		stats.getMaxSpeed().unmodify(id);
		stats.getAcceleration().unmodify(id);
		stats.getDeceleration().unmodify(id);
		stats.getMaxTurnRate().unmodify(id);
		stats.getTurnAcceleration().unmodify(id);
	}

	@Override
	public StatusData getStatusData(int index, State state, float effectLevel) {
		if (index == 0) {
			return new StatusData("", false);
		}
		return null;
	}
}
