package data.scripts;

import com.fs.starfarer.api.BaseModPlugin;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.PluginPick;
import com.fs.starfarer.api.campaign.CampaignPlugin;
import com.fs.starfarer.api.campaign.SectorAPI;
import com.fs.starfarer.api.combat.MissileAIPlugin;
import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.impl.campaign.procgen.ProcgenUsedNames;
import data.scripts.ai.GDLMHiveMissileAI;
import data.scripts.ai.GDLMPDMissileAI;
import data.scripts.world.Mosta.GudalanmuSL;
import com.fs.starfarer.api.impl.campaign.shared.SharedData;
import org.dark.shaders.light.LightData;
import org.dark.shaders.util.ShaderLib;
import org.dark.shaders.util.TextureData;

public class GDLMModPlugin extends BaseModPlugin {

	public static void newGenerate(SectorAPI sector) {
		ProcgenUsedNames.notifyUsed("Gudalanmu");
		new GudalanmuSL().generate(sector);
	}

	@Override
	public void onNewGame() {
		if (Global.getSettings().getModManager().isModEnabled("nexerelin")) {
			GDLMSetNEXSettings.generate(Global.getSector());
		} else {
			newGenerate(Global.getSector());
		}
		SharedData.getData().getPersonBountyEventData().addParticipatingFaction("Gudalanmu");

	}

	public static boolean hasGraphicsLib;

	@Override
	public void onApplicationLoad() {
		hasGraphicsLib = Global.getSettings().getModManager().isModEnabled("shaderLib");

		if (hasGraphicsLib) {
			ShaderLib.init();
			LightData.readLightDataCSV("data/lights/GL_light_data.csv");
			TextureData.readTextureDataCSV("data/lights/GL_texture_data.csv");
		}
	}

	@Override
	public PluginPick<MissileAIPlugin> pickMissileAI(MissileAPI missile, ShipAPI launchingShip) {
		switch (missile.getProjectileSpecId()) {
			case "GLSI_EM_MISSILE":
				return new PluginPick<MissileAIPlugin>(new GDLMHiveMissileAI(missile, launchingShip), CampaignPlugin.PickPriority.MOD_SPECIFIC);
			case "GLSI_SIAMM_MISSILE":
				return new PluginPick<MissileAIPlugin>(new GDLMPDMissileAI(missile, launchingShip), CampaignPlugin.PickPriority.MOD_SPECIFIC);

		}
		return null;
	}
} 