//by Tartiflette, for the guiding part
//by a111164
package data.scripts.ai;

import com.fs.starfarer.api.combat.*;
import data.scripts.utils.GDLMTool;
import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.VectorUtils;
import org.lazywizard.lazylib.combat.AIUtils;
import org.lwjgl.util.vector.Vector2f;

public class GDLMHiveMissileAI implements GuidedMissileAI, MissileAIPlugin {

	private CombatEntityAPI target;
	private Vector2f targetPo = new Vector2f(0f, 0f);
	private ShipAPI lunchShip;
	private MissileAPI missile;

	private STATUS status = new SEARCHING();
	private float timer;

	private static final float ROUGH_SEARCH_RANGE = 2000f;
	private static final float SIDE_RANGE = 100f;
	private static final float DEDICATE_AIM_RANGE = 500f;//if dist closer than this, missile will go straight, should larger than SIDE_RANGE
	private static final float ENGINE_START_TIME = 0.5f;//control after when missile start searching target
	private static final float SWING_ANGLE = 5f;//control swing angle
	private static final float MAX_SWING_ANGLE = 20;//control max swing angle

	public GDLMHiveMissileAI(MissileAPI missile, ShipAPI lunchShip) {
		this.missile = missile;
		this.lunchShip = lunchShip;
	}

	@Override
	public CombatEntityAPI getTarget() {
		return target;
	}

	@Override
	public void setTarget(CombatEntityAPI newTarget) {
		this.target = newTarget;
	}

	@Override
	public void advance(float amount) {
		if (missile == null || lunchShip == null) {
			return;
		}

		timer = timer + amount;

		//init no searching time
		if (timer < ENGINE_START_TIME) {
			missile.giveCommand(ShipCommand.ACCELERATE);
			return;
		}

		//ready to end check
		if (timer > missile.getMaxFlightTime()) {
			status = new READY_TO_END();
			return;
		}

		if (target != null) {
			//if lunchShip has target, set missile target to it
			if (lunchShip.getShipTarget() != null && target != lunchShip.getShipTarget()) {
				target = lunchShip.getShipTarget();
				status = new SWING_TO_TARGET();
				return;
			}

			//if target is dead, in to searching mod
			if (!((ShipAPI) target).isAlive()) {
				target = null;
				status = new SEARCHING();
				return;
			}
		} else {
			if (!(status instanceof SEARCHING)) {
				status = new SEARCHING();
			}
		}

		status.advance(amount);

	}

	private class SEARCHING implements STATUS {
		float timer = 1f;

		@Override
		public void advance(float amount) {
			timer = timer + amount;

			//changing status if we had a target
			if (target != null && target != lunchShip && ((ShipAPI) target).isAlive()) {
				status = new SWING_TO_TARGET();
				return;
			}

			missile.giveCommand(ShipCommand.ACCELERATE);

			if (timer > 0.5) {
				timer = 0f;
				target = lunchShip;

				//find the nearest fighter
				float dist = 0f;
				float minDist = ROUGH_SEARCH_RANGE;
				for (ShipAPI s : AIUtils.getNearbyEnemies(missile, ROUGH_SEARCH_RANGE)) {

					if ((s.isCapital() || s.isCruiser() || s.isDestroyer() || s.isFrigate()) || s.isFighter()) {
						dist = MathUtils.getDistance(s, (CombatEntityAPI) missile);
						float turnTime = GDLMTool.getTimeNeedToTurn(missile.getFacing(), VectorUtils.getAngle(missile.getLocation(), s.getLocation()), missile.getAngularVelocity(), missile.getAcceleration(), missile.getAcceleration(), missile.getMaxTurnRate());
						float fiyTime = dist / missile.getMaxSpeed();
						if (turnTime < fiyTime && dist < minDist) {
							minDist = dist;
							target = (CombatEntityAPI) s;
						}
					}
				}

			}

		}
	}

	private class SWING_TO_TARGET implements STATUS {
		private float timer;

		@Override
		public void advance(float amount) {
			timer = timer + amount;

			//if target is null, return to searching mod
			if (target == null || !((ShipAPI) target).isAlive()) {
				target = null;
				status = new SEARCHING();
				return;
			}

			if (((ShipAPI) target).isFighter()) {
				status = new STRAIGHT_TO_TARGET();
			}

			float targetToMissileAngle = VectorUtils.getAngle(target.getLocation(), missile.getLocation());
			if (MathUtils.getShortestRotation(target.getFacing(), targetToMissileAngle) > 0) {
				targetPo = GDLMTool.getExtendedLocationFromPoint(target.getLocation(), GDLMTool.getAngleCalculation(target.getFacing(), 90f), target.getCollisionRadius() + SIDE_RANGE);
			} else {
				targetPo = GDLMTool.getExtendedLocationFromPoint(target.getLocation(), GDLMTool.getAngleCalculation(target.getFacing(), -90f), target.getCollisionRadius() + SIDE_RANGE);
			}

			float dist = MathUtils.getDistance(missile.getLocation(), target.getLocation());

			missile.giveCommand(ShipCommand.ACCELERATE);
			swingToTarget(missile, targetPo);

			//Global.getCombatEngine().addFloatingText(targetPo,"turnRate", 20f ,new Color (0, 100, 200, 240),target, 0.25f, 120f);

			//if to close, straight to target
			if (dist < DEDICATE_AIM_RANGE + target.getCollisionRadius()) {
				status = new STRAIGHT_TO_TARGET();
				return;
			}
		}

	}

	private class STRAIGHT_TO_TARGET implements STATUS {
		private float timer;

		@Override
		public void advance(float amount) {
			timer = timer + amount;

			//if target is null, return to searching mod
			if (target == null || !((ShipAPI) target).isAlive()) {
				target = null;
				status = new SEARCHING();
				return;
			}

			targetPo = AIUtils.getBestInterceptPoint(missile.getLocation(), missile.getMoveSpeed(), target.getLocation(), target.getVelocity());
			if (targetPo == null) {
				targetPo = target.getLocation();
			}

			if (Math.abs(MathUtils.getShortestRotation(missile.getFacing(), VectorUtils.getAngle(missile.getLocation(), targetPo))) < 90f) {
				missile.giveCommand(ShipCommand.ACCELERATE);
			} else {
				missile.giveCommand(ShipCommand.DECELERATE);
			}
			GDLMTool.moveToAngle(missile, VectorUtils.getAngle(missile.getLocation(), targetPo), amount);
		}

	}

	private void swingToTarget(MissileAPI m, Vector2f toTargetPo) {

		float dist = MathUtils.getShortestRotation(m.getFacing(), VectorUtils.getAngle(m.getLocation(), toTargetPo));
		float maxSwingTurnRate = (float) Math.sqrt(2f * missile.getTurnAcceleration() * MAX_SWING_ANGLE);

		if (dist > SWING_ANGLE) {
			m.giveCommand(ShipCommand.TURN_LEFT);
		}

		if (dist < -SWING_ANGLE) {
			m.giveCommand(ShipCommand.TURN_RIGHT);
		}

		if (Math.abs(dist) <= SWING_ANGLE) {
			if (Math.abs(m.getAngularVelocity()) < maxSwingTurnRate) {
				if (m.getAngularVelocity() > 0) {
					m.giveCommand(ShipCommand.TURN_LEFT);
				} else {
					m.giveCommand(ShipCommand.TURN_RIGHT);
				}
			} else {
				if (m.getAngularVelocity() > 0) {
					m.giveCommand(ShipCommand.TURN_RIGHT);
				} else {
					m.giveCommand(ShipCommand.TURN_LEFT);
				}
			}

		}

	}

	private class READY_TO_END implements STATUS {
		@Override
		public void advance(float amount) {

		}
	}

	private interface STATUS {
		void advance(float amount);
	}

}