//by Tartiflette, for the guiding part
//by a111164
package data.scripts.ai;

import com.fs.starfarer.api.combat.*;
import data.scripts.utils.GDLMTool;
import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.VectorUtils;
import org.lazywizard.lazylib.combat.AIUtils;
import org.lwjgl.util.vector.Vector2f;

import java.util.List;

public class GDLMPDMissileAI implements GuidedMissileAI, MissileAIPlugin {

	private CombatEntityAPI target;
	private Vector2f targetPo = new Vector2f(0f, 0f);
	private ShipAPI lunchShip;
	private MissileAPI missile;

	private STATUS status = new SEARCHING();
	private float timer;

	private static final float ROUGH_SEARCH_RANGE = 1000f;
	private static final float ENGINE_START_TIME = 0f;//control after when missile start searching target
	private static final float SEARCHING_SLOW_SPEED_MULT = 0.35f;

	public GDLMPDMissileAI(MissileAPI missile, ShipAPI lunchShip) {
		this.missile = missile;
		this.lunchShip = lunchShip;
	}

	@Override
	public CombatEntityAPI getTarget() {
		return target;
	}

	@Override
	public void setTarget(CombatEntityAPI newTarget) {
		this.target = newTarget;
	}

	@Override
	public void advance(float amount) {
		if (missile == null || lunchShip == null) {
			return;
		}

		timer = timer + amount;

		//init no searching time
		if (timer < ENGINE_START_TIME) {
			missile.giveCommand(ShipCommand.ACCELERATE);
			return;
		}

		//ready to end check
		if (timer > missile.getMaxFlightTime()) {
			//Global.getCombatEngine().addFloatingText(missile.getLocation(), "search", 20f ,new Color (0, 100, 200, 240),missile, 0.25f, 120f);

			status = new READY_TO_END();
			return;
		}

		// null check
		if (target == null && !(status instanceof SEARCHING)) {
			status = new SEARCHING();
		}

		status.advance(amount);

	}

	private class SEARCHING implements STATUS {

		@Override
		public void advance(float amount) {

			//Global.getCombatEngine().addFloatingText(missile.getLocation(), "search", 20f ,new Color (0, 100, 200, 240),missile, 0.25f, 120f);

			//changing status if we had a target
			if (target != null && target != lunchShip && target.getHitpoints() > 0f) {
				status = new STRAIGHT_TO_TARGET();
				return;
			}

			if (GDLMTool.Velocity2Speed(missile.getVelocity()).getY() < missile.getMaxSpeed() * SEARCHING_SLOW_SPEED_MULT) {
				missile.giveCommand(ShipCommand.ACCELERATE);
			} else {
				missile.giveCommand(ShipCommand.DECELERATE);
			}

			if (missile.getAngularVelocity() > 1) {
				missile.giveCommand(ShipCommand.TURN_RIGHT);
			}
			if (missile.getAngularVelocity() < -1) {
				missile.giveCommand(ShipCommand.TURN_LEFT);
			}

			target = null;
			//find a random missile in range
			float dist = 0f;
			float minDist = ROUGH_SEARCH_RANGE;
			List<MissileAPI> missiles = AIUtils.getNearbyEnemyMissiles(missile, ROUGH_SEARCH_RANGE);
			if (missiles.size() != 0) {
				MissileAPI m = missiles.get((int) (Math.random() * missiles.size()));
				if (m.getCollisionClass() != CollisionClass.NONE) {
					target = m;
					status = new STRAIGHT_TO_TARGET();
				}
			}
			//find the nearest fighter if not missiles
			else {
				dist = 0f;
				minDist = ROUGH_SEARCH_RANGE;
				for (ShipAPI s : AIUtils.getNearbyEnemies(missile, ROUGH_SEARCH_RANGE)) {

					if (s.isFighter()) {
						dist = MathUtils.getDistance(s, (CombatEntityAPI) missile);
						float turnTime = GDLMTool.getTimeNeedToTurn(missile.getFacing(), VectorUtils.getAngle(missile.getLocation(), s.getLocation()), missile.getAngularVelocity(), missile.getAcceleration(), missile.getAcceleration(), missile.getMaxTurnRate());
						float fiyTime = dist / missile.getMaxSpeed();
						if (turnTime < fiyTime && dist < minDist) {
							minDist = dist;
							target = (CombatEntityAPI) s;
						}
					}
				}
			}

		}
	}

	private class STRAIGHT_TO_TARGET implements STATUS {

		@Override
		public void advance(float amount) {

			//if target is null, return to searching mod
			if (target instanceof MissileAPI) {

				if (target == null || ((MissileAPI) target).getFlightTime() > ((MissileAPI) target).getMaxFlightTime() || ((MissileAPI) target).didDamage() || target.getHullLevel() <= 0f) {
					target = null;
					status = new SEARCHING();
					return;
				}

			}

			if (target instanceof ShipAPI) {
				if (!((ShipAPI) target).isAlive()) {
					target = null;
					status = new SEARCHING();
					return;
				}
			}

			targetPo = AIUtils.getBestInterceptPoint(missile.getLocation(), missile.getMoveSpeed(), target.getLocation(), target.getVelocity());
			if (targetPo == null) {
				targetPo = target.getLocation();
			}

			if (Math.abs(MathUtils.getShortestRotation(missile.getFacing(), VectorUtils.getAngle(missile.getLocation(), targetPo))) < 90f) {
				missile.giveCommand(ShipCommand.ACCELERATE);
			} else {
				missile.giveCommand(ShipCommand.DECELERATE);
			}
			GDLMTool.moveToAngle(missile, VectorUtils.getAngle(missile.getLocation(), targetPo), amount);
		}

	}

	private class READY_TO_END implements STATUS {
		@Override
		public void advance(float amount) {

		}
	}

	private interface STATUS {
		void advance(float amount);
	}

}