package data.scripts.plugin;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.*;
import com.fs.starfarer.api.input.InputEventAPI;
import org.lazywizard.lazylib.VectorUtils;

import java.util.*;

public class GL_WeaponPlugin extends BaseEveryFrameCombatPlugin {
	private static final String DATA_KEY = "GL_WeaponPlugin";

	private CombatEngineAPI engine;

	@Override
	public void advance(float amount, List<InputEventAPI> events) {
		if (engine == null || engine.isPaused()) {
			return;
		}

		final LocalData localData = (LocalData) engine.getCustomData().get(DATA_KEY);
		final List<DamagingProjectileAPI> toRemove = localData.toRemove;

		for (DamagingProjectileAPI projectile : engine.getProjectiles()) {
			if (!engine.isEntityInPlay(projectile)) {
				continue;
			}

			String projectileID = projectile.getProjectileSpecId();
			if (projectileID == null) {
				continue;
			}

			if (projectile.getElapsed() > 0) {
				continue;
			}

			switch (projectileID) {
				/*
				每一个 case 后面都写你希望让它霰射的弹体的id，如有需要可以自行继续添加更多 case
				霰射的弹体必须满足：
					"spawnType":"BALLISTIC",
					"collisionClass":"PROJECTILE_FF",
					"collisionClassByFighter":"PROJECTILE_FIGHTER",
				如不满足请自行修改弹体的.proj文件
				 */
				case "GL_KRG_shot":
					// 第一个数字是角度，这里的10表示正负10度，如果写20就是正负20度，不应该大于180
					// 第二个数字表示速度的小幅随机，0.1的意思是弹体速度变为90%~110%之间的随机数，如果写0.2则弹体速度为80~120%，不应该超过1也不应该为负
					makeShotgun(projectile, 10f, 0.1f);
					break;
				case "GL_DKRG_shot":
					makeShotgun(projectile, 15f, 0.2f);
					break;
				case "your_proj_id_3":
					makeShotgun(projectile, 5f, 0.1f);
					break;
				case "your_proj_id_4":
					makeShotgun(projectile, 7f, 0.2f);
					break;
				case "your_proj_id_5":
					makeShotgun(projectile, 12f, 0.1f);
					break;
			}
		}

		if (!toRemove.isEmpty()) {
			for (DamagingProjectileAPI remove : toRemove) {
				engine.removeEntity(remove);
			}
		}
		toRemove.clear();
	}

	@Override
	public void init(CombatEngineAPI engine) {
		this.engine = engine;
		Global.getCombatEngine().getCustomData().put(DATA_KEY, new LocalData());
	}

	public static final class LocalData {
		private final List<DamagingProjectileAPI> toRemove = new ArrayList<>();
	}

	private static void makeShotgun(DamagingProjectileAPI proj, float angleChange, float speedChange) {
		if (Math.abs(angleChange) > 180f) angleChange = 180f;
		if (speedChange > 1f) speedChange = 1f;
		if (speedChange < 0f) speedChange = 0;

		float facings = (float) Math.pow((float) Math.random(), 2) * angleChange * ((float) Math.random() < 0.5f ? 1f : -1f);
		float vels = (1f - speedChange) + (float) Math.random() * 2f * speedChange;

		proj.setFacing(proj.getFacing() + facings);
		proj.getVelocity().scale(vels);
		VectorUtils.rotate(proj.getVelocity(), facings);
	}
}