package data.scripts.weapons;

import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.EveryFrameWeaponEffectPlugin;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.WeaponAPI;
import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.VectorUtils;
import org.lwjgl.util.vector.Vector2f;

public class GDLM_Engine implements EveryFrameWeaponEffectPlugin {

	private ShipAPI ship;
	private WeaponAPI w;
	private float amount;
	private CombatEngineAPI engine;

	private float toAngle;
	private float arcAngle;

	private static float TUNRING_SPEED = 20f;
	private static float MAX_ANGLE = 100f;

	@Override
	public void advance(float amount, CombatEngineAPI engine, WeaponAPI weapon) {
		this.amount = amount;
		this.w = weapon;
		this.ship = weapon.getShip();
		this.engine = engine;

		if (engine.isPaused() || weapon.getSlot().isHidden() || !ship.isAlive()) {
			return;
		}

		arcAngle = getAngleCalculation(weapon.getArcFacing(), ship.getFacing());

		if (MathUtils.getShortestRotation(VectorUtils.getAngle(ship.getLocation(), weapon.getLocation()), ship.getFacing()) > 0) {
			if (ship.getEngineController().isTurningLeft()) {
				toAngle = limitToTop(toAngle + TUNRING_SPEED * amount, MAX_ANGLE, 0);
			} else {
				toAngle = limitToTop(toAngle - TUNRING_SPEED * amount, MAX_ANGLE, 0);
			}

		} else {
			if (ship.getEngineController().isTurningRight()) {
				toAngle = limitToTop(toAngle - TUNRING_SPEED * amount, 0, -MAX_ANGLE);
			} else {
				toAngle = limitToTop(toAngle + TUNRING_SPEED * amount, 0, -MAX_ANGLE);
			}
		}

		w.setCurrAngle(arcAngle + toAngle);

	}

	public static Vector2f getExtendedLocationFromPoint(Vector2f point, float facing, float dist) {
		float xAxis = (float) Math.cos(Math.toRadians(facing)) * dist;
		float yAxis = (float) Math.sin(Math.toRadians(facing)) * dist;
		return new Vector2f(point.getX() + xAxis, point.getY() + yAxis);
	}

	//+ = turnLeft, - = turn right
	public static float getAngleCalculation(float originAngle, float addAngle) {
		float finalAngle = originAngle + addAngle;

		while (addAngle > 360) {
			addAngle = addAngle - 360;
		}

		while (addAngle < -360) {
			addAngle = addAngle + 360;
		}

		if (finalAngle > 360f) {
			return finalAngle - 360f;
		}

		if (finalAngle < 0f) {
			return 360f + finalAngle;
		}
		return finalAngle;
	}

	public float limitToTop(float input, float limitMax, float limitMin) {
		if (input > limitMax) {
			return limitMax;
		}
		if (input < limitMin) {
			return limitMin;
		}
		return input;
	}

}

        	         	       	       	 
