package data.scripts.weapons;

import com.fs.starfarer.api.combat.BeamAPI;
import com.fs.starfarer.api.combat.BeamEffectPlugin;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.CombatEntityAPI;

import java.awt.*;

public class GDLM_FocusedBeamEffect implements BeamEffectPlugin {
	private float clock = 0f;
	private boolean init = true;
	private Color origColor = null;

	@Override
	public void advance(float amount, CombatEngineAPI engine, BeamAPI beam) {
		if (origColor == null) {
			origColor = beam.getFringeColor();
		}
		if (beam.getBrightness() <= 0.01f) {
			return;
		}
		clock += amount;
		Color newColor;
		if (clock < 1f && !init) {
			newColor = colorBlenderWithAlpha(Color.white, origColor, clock);
		} else if (clock < 4f) {
			newColor = origColor;
		} else if (clock < 5f) {
			newColor = colorBlenderWithAlpha(origColor, Color.white, clock - 4f);
		} else {
			newColor = Color.white;
			CombatEntityAPI target = beam.getDamageTarget();
			if (target != null) {
				float damage = beam.getWeapon().getDamage().getDamage();
				engine.applyDamage(target, beam.getTo(), damage * amount * 0.3f, beam.getDamage().getType(), 0f, false, true, beam.getSource());
			}
			init = false;
			clock = 0f;
		}
		beam.setFringeColor(newColor);
	}

	private static Color colorBlenderWithAlpha(Color basicColor, Color targetColor, float effectLevel) {
		effectLevel = Math.max(Math.min(effectLevel, 1f), 0f);
		float r = basicColor.getRed() * (1f - effectLevel) + targetColor.getRed() * effectLevel;
		float g = basicColor.getGreen() * (1f - effectLevel) + targetColor.getGreen() * effectLevel;
		float b = basicColor.getBlue() * (1f - effectLevel) + targetColor.getBlue() * effectLevel;
		float a = basicColor.getAlpha() * (1f - effectLevel) + targetColor.getAlpha() * effectLevel;
		return new Color((int) r, (int) g, (int) b, (int) a);
	}
}