package data.scripts.weapons;

import com.fs.starfarer.api.combat.BeamAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.BeamEffectPlugin;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.util.WeightedRandomPicker;
import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.VectorUtils;

import java.util.Random;

public class GDLM_RayScatteringEffect implements BeamEffectPlugin {
	@Override
	public void advance(float amount, CombatEngineAPI engine, BeamAPI beam) {
		if (beam.getDamageTarget() != null) {
			if (beam.getDamageTarget() instanceof ShipAPI) {
				ShipAPI target = (ShipAPI) beam.getDamageTarget();
				if (target.getShield() != null) {
					if (target.getShield().isOn() && target.getShield().isWithinArc(beam.getTo())) {
						return;
					}
				}
			}
		}
		Random random = new Random();
		if (random.nextFloat() < amount * 20f) { //每秒大约20次
			float beamAngle = VectorUtils.getAngle(beam.getFrom(), beam.getTo());//光束角度（从头到尾）
			float emitterAngle = MathUtils.clampAngle(beamAngle - 180f);//角度翻转
			float spreadAngle = (float) (random.nextGaussian() * 85f);//高斯分布，如果想调整密度就改这个45.
			if (spreadAngle > 90f) {//避免超过90
				spreadAngle = 90f;
			} else if (spreadAngle < -90f) {//避免超过-90
				spreadAngle = -90f;
			}//随机角度-90~90，这样就能在扇形180内了
			float resultAngle = emitterAngle + spreadAngle;//最终弹丸角度
			WeightedRandomPicker<String> fakeWeaponIDs = new WeightedRandomPicker<>(random);
			fakeWeaponIDs.add("GL_RFW_1");
			fakeWeaponIDs.add("GL_RFW_2");
			fakeWeaponIDs.add("GL_RFW_3");//以此类推

			engine.spawnProjectile(beam.getSource(), beam.getWeapon(), fakeWeaponIDs.pick(),//随机从这些id中选择
					MathUtils.getPointOnCircumference(beam.getTo(), random.nextFloat() * 30f + 10f,//生成的弹丸得远离光束击打的地方，此处区间距离为10~40
							resultAngle),//这个方法取以光束终点为中心，指定角度，指定距离的点
					resultAngle,//弹丸生成角度
					null);//舰船矢量，一般不需要
		}
	}
}

