package data.scripts.world.Mosta.MostaSystem;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.JumpPointAPI;
import com.fs.starfarer.api.campaign.OrbitAPI;
import com.fs.starfarer.api.campaign.PlanetAPI;
import com.fs.starfarer.api.campaign.SectorAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.SectorGeneratorPlugin;
import com.fs.starfarer.api.campaign.StarSystemAPI;
import com.fs.starfarer.api.campaign.SpecialItemData;
import com.fs.starfarer.api.campaign.econ.EconomyAPI;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.campaign.econ.MarketConditionAPI;
import com.fs.starfarer.api.impl.campaign.ids.Items;
import com.fs.starfarer.api.impl.campaign.procgen.NebulaEditor;
import com.fs.starfarer.api.impl.campaign.procgen.StarAge;
import com.fs.starfarer.api.impl.campaign.terrain.HyperspaceTerrainPlugin;
import com.fs.starfarer.api.util.Misc;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Arrays;

public class GDLM implements SectorGeneratorPlugin {

	@Override
	public void generate(SectorAPI sector) {
		StarSystemAPI system = sector.createStarSystem("Mosta");
		system.getLocation().set(10000, 10000);
		system.setBackgroundTextureFilename("graphics/backgrounds/background2.jpg");//graphic path
		PlanetAPI star = system.initStar("Mosta_star", //ID
				"star_yellow", //type
				600f, //恒星默认半径（在默认缩放比率下）
				450,//日冕半径 从恒星边缘开始计算
				10f,//太阳风燃烧水平
				1f,// 耀斑概率
				3f);// CR损失率
		system.setLightColor(new Color(255, 255, 255));
		PlanetAPI Yester = system.addPlanet("Yester", star, "Yester", "planet_Yester", 60, 120, 5500, 265); //添加星球 参数分别为 ID,轴心,展示名,星球类型,星球倾斜角度,轨道半径,轨道天数
		PlanetAPI YesterI = system.addPlanet("YesterI", star, "Valata", "planet_Valata", 45, 120, 6800, 475);
		PlanetAPI YesterII = system.addPlanet("YesterII", star, "Kuvataka", "barren", 80, 80, 1200, 125);
		Yester.setCustomDescriptionId("Yester_name"); //描述定义于 data/strings/descriptions.csv
		YesterI.setCustomDescriptionId("Valata_name");
		YesterII.setCustomDescriptionId("Kuvataka_name");
		//Yester.setInteractionImage("illustrations", "bombard_saturation_result"); //设置该星球的互动展示图片

		SectorEntityToken Yester_station = system.addCustomEntity("Yester_station", "中枢空间站", "station_GL_Central", "Gudalanmu"); //添加空间站 参数列表分别为 ID,展示名,类型(可自定义),所属势力ID
		Yester_station.setCircularOrbitPointingDown(star, 45, 3100, 180); //设置轨道 参数列表分别为 轴心,角度,轨道半径,轨道周期
		Yester_station.setCustomDescriptionId("Yester_station"); //设置自定义描述ID

		MarketAPI YesterMarket = addMarketplace("Gudalanmu", Yester, 7,//population写多少，这就写多少
				new ArrayList(Arrays.asList(new String[]{"population_7",//环境
						"habitable", "hot", "volatiles_trace", "ore_rich", "GDLMcolonialautomaticfactory", "rare_ore_moderate", "farmland_poor"})), new ArrayList(Arrays.asList(new String[]{"population",//设施
						"farming", "megaport", "orbitalworks", "mining", "starfortress_high", "heavybatteries", "highcommand"})), new ArrayList(Arrays.asList(new String[]{"open_market", "black_market", "generic_military", "storage"})),//有什么子市场
				0.2f, false, true, true);
		//税率相关因素
		Yester_station.setMarket(YesterMarket);

		MarketAPI YesterIMarket = addMarketplace("Gudalanmu", YesterI, 5, new ArrayList(Arrays.asList(new String[]{"population_5", "habitable", "organics_abundant", "volatiles_diffuse", "inimical_biosphere", "rare_ore_abundant", "farmland_poor"})), new ArrayList(Arrays.asList(new String[]{"population", "refining", "megaport", "mining", "battlestation_high", "heavybatteries", "militarybase"})), new ArrayList(Arrays.asList(new String[]{"open_market", "black_market", "storage"})), 0.2f, false, true, false);

		MarketAPI Yester_stationMarket = addMarketplace("Gudalanmu", Yester_station, 5, new ArrayList(Arrays.asList(new String[]{"population_5", "ore_sparse", "free_market"})), new ArrayList(Arrays.asList(new String[]{"population", "fuelprod", "megaport", "mining", "battlestation_high", "heavybatteries", "militarybase"})), new ArrayList(Arrays.asList(new String[]{"open_market", "black_market", "generic_military", "storage"})), 0.2f, false, true, false);

		SectorEntityToken relay = system.addCustomEntity("corvus_relay", "Mosta 通讯中继站", "comm_relay", "Gudalanmu");
		relay.setCircularOrbitPointingDown(system.getEntityById("Mosta_star"), 245 - 60, 7500, 200);

		SectorEntityToken array1 = system.addCustomEntity("sensor_array_makeshift", "Mosta 传感器阵列", "sensor_array_makeshift", "Gudalanmu");
		array1.setCircularOrbitPointingDown(system.getEntityById("Mosta_star"), 245 - 60 - 10, 9500, 400);

		SectorEntityToken buoy1 = system.addCustomEntity("nav_buoy_makeshift", "Mosta 导航信标", "nav_buoy_makeshift", "Gudalanmu");
		buoy1.setCircularOrbitPointingDown(system.getEntityById("Mosta_star"), 245 - 60 - 20, 11500, 400);

		JumpPointAPI jumpPoint1 = Global.getFactory().createJumpPoint("GL_JumpPoint1", "SwarmDoor");
		OrbitAPI orbit = Global.getFactory().createCircularOrbit(Yester, 220, 2000, 365);
		jumpPoint1.setOrbit(orbit);
		jumpPoint1.setRelatedPlanet(Yester);
		jumpPoint1.setStandardWormholeToHyperspaceVisual();
		system.addEntity(jumpPoint1);

		//添加小行星带 参数列表为 轴心,小行星数量,轨道半径,宽度,最小周期天数,最大周期天数
		system.addAsteroidBelt(star, 130, 3000, 256, 200, 300);
		system.addAsteroidBelt(star, 130, 3600, 256, 200, 300);
		system.addAsteroidBelt(star, 110, 3700, 256, 200, 300);
		system.addAsteroidBelt(star, 120, 3500, 256, 200, 300);
		system.addAsteroidBelt(star, 130, 4100, 256, 200, 300);

		//添加环带 参数列表分别为 轴心,类型,环的ID,纹理宽度,纹理序号,颜色,这一个参数不太清楚什么意思,环带半径,周期天数
		system.addRingBand(star, "misc", "rings_ice0", 300f, 4, Color.white, 1000f, 3600, 365f);
		system.addRingBand(star, "misc", "rings_dust0", 600f, 4, Color.white, 1000f, 3000, 365f);

		//用png添加星云
		SectorEntityToken Mosta_Nebula = Misc.addNebulaFromPNG("graphics/terrain/nebula01.png", 0f, 0f, system, "terrain", "nebula_blue", 4, 4, StarAge.AVERAGE);
		system.autogenerateHyperspaceJumpPoints(true, true); //设置自动生成跳跃点

		cleanup(system);
	}

	public static MarketAPI addMarketplace(String factionID, SectorEntityToken primaryEntity, int size, ArrayList<String> marketConditions, ArrayList<String> marketIndustries, ArrayList<String> submarkets, float tarrif, boolean hasCore, boolean hasForge, boolean hasSynchrotron) {
		EconomyAPI globalEconomy = Global.getSector().getEconomy();
		String planetID = primaryEntity.getId();
		String marketID = planetID;
		MarketAPI newMarket = Global.getFactory().createMarket(marketID, primaryEntity.getName(), size);
		newMarket.setFactionId(factionID);
		newMarket.setPrimaryEntity(primaryEntity);
		newMarket.getTariff().modifyFlat("generator", tarrif);
		newMarket.setSurveyLevel(MarketAPI.SurveyLevel.FULL);
		if (null != submarkets) {
			for (String market : submarkets) {
				newMarket.addSubmarket(market);
			}
		}
		if (null != marketConditions) {
			for (String condition : marketConditions) {
				newMarket.addCondition(condition);
				if (hasForge && condition.equals("free_market")) {
					newMarket.setFreePort(true);
				}
			}
		}
		if (null != newMarket.getConditions() && !newMarket.getConditions().isEmpty()) {
			for (MarketConditionAPI condition : newMarket.getConditions()) {
				condition.setSurveyed(true);
			}
		}
		if (null != marketIndustries) {
			for (String industry : marketIndustries) {
				newMarket.addIndustry(industry);
				if (hasCore) {
					if ((float) Math.random() < ((float) size * 0.1f) - 0.5f) {//fuc it
						newMarket.getIndustry(industry).setAICoreId("alpha_core");
					} else if ((float) Math.random() < ((float) size * 0.1f) - 0.25f) {
						newMarket.getIndustry(industry).setAICoreId("beta_core");
					} else {
						newMarket.getIndustry(industry).setAICoreId("gamma_core");
					}
				}
				if (hasForge && industry.equals("orbitalworks")) {
					newMarket.getIndustry(industry).setSpecialItem(new SpecialItemData(Items.PRISTINE_NANOFORGE, null));
				}
				if (hasSynchrotron && industry.equals("fuelprod")) {
					newMarket.getIndustry(industry).setSpecialItem(new SpecialItemData(Items.SYNCHROTRON, null));
				}
			}
		}
		globalEconomy.addMarket(newMarket, true);
		primaryEntity.setMarket(newMarket);
		primaryEntity.setFaction(factionID);
		return newMarket;
	}

	public static void cleanup(StarSystemAPI system) {
		HyperspaceTerrainPlugin plugin = (HyperspaceTerrainPlugin) Misc.getHyperspaceTerrain().getPlugin();
		NebulaEditor editor = new NebulaEditor(plugin);
		float minRadius = plugin.getTileSize() * 2f;

		float radius = system.getMaxRadiusInHyperspace();
		editor.clearArc(system.getLocation().x, system.getLocation().y, 0, radius + minRadius * 0.5f, 0, 360f);
		editor.clearArc(system.getLocation().x, system.getLocation().y, 0, radius + minRadius, 0, 360f, 0.25f);
	}

}
